<?php
if (!defined('ABSPATH')) {
	exit;
}

date_default_timezone_set(get_option('timezone_string'));

/**
 * 前端采集页面功能
 * 
 * @since 2019.10.1
 */
class Grab
{

	/** JPOST类 */
	private $JPost;

	/** 输出HTML */
	private $output_html = '';

	private $_jpost_i;
	private $cPage;
	private $debug;

	/** 请求路径 */
	private $handle_url = '';

	/** 请求参数 */
	private $handle_args = array();

	/** 获取渠道 */
	private $get_ele_method = 'selector';

	/**采集页HTML */
	private $html;

	/** DOMXPath */
	private $dom;

	public function __construct()
	{
		add_action('template_redirect', array($this, 'jpost_grab'));
		add_filter('query_vars', array($this, 'jpost_query_vars'));
		add_action('wp_ajax_jpost_iframe', array($this, 'iframe_callback'));

		$this->init();
	}

	public function init()
	{
		$this->JPost = new JPost();
	}

	/**
	 * 添加p标签
	 * 
	 * @since 2021.5.1
	 */
	private function jpost_output($str)
	{
		return '<p>' . $str . '</p>';
	}

	/**
	 * 添加<h1></h1>标签
	 * 
	 * @since 2021.5.1
	 */
	private function jpost_h1($str)
	{
		return '<h1>' . $str . '</h1>';
	}

	/**
	 * 中文转化成UTF-8字符集
	 * 
	 * @since 2021.5.1
	 */
	private function jpost_char2UTF8($str)
	{
		if (!extension_loaded('mbstring')) {
			wp_die(__('请安装 ` mbstring ` 扩展', 'wp-jpost'));
		}
		/* 判断是否不是UTF-8编码，如果不是UTF-8编码，则转换为UTF-8编码 */
		if (mb_detect_encoding($str, "UTF-8, ISO-8859-1, GBK") != "UTF-8") {
			return mb_convert_encoding($str, "UTF-8", 'GBK,GB2312,auto');
		} else {
			return $str;
		}
	}

	/**
	 * JAVASCRIPT跳转
	 * 
	 * @since 2021.5.1
	 */
	private function jpost_meta_go($url = '', $time = '0')
	{
		if ($this->debug && $url)
			return;
		if (!$url && !$time)
			$time = '6';
		$url = esc_url_raw($url);
		$div = $url ? 'j' . md5($url) : 'showSeconds';
		// printf( '<meta http-equiv="refresh" content="%s; url=%s">' , $time , $url );
		printf(__('正在跳转下一个网址:) <a href="%s" rel="nofollow">%s</a>', 'wp-jpost'), $url, $url);
		printf('<h2 id="%s" style="text-align:center;"></h2>', $div);
		printf('<script type="text/javascript">');
		printf('	var t%s=%s; ', $div, $url ? $time : $time * 2);
		printf('	setInterval("jreload%s()",1000);', $div);
		printf('	function jreload%s(){  ', $div);
		printf('		if(t%s==0){ location="%s"; }', $div, $url ? $url : '"+window.location.href+"&random="+Math.random()+"');
		printf('		document.getElementById("%s").innerHTML=""+t%s+"%s";', $div, $div, __(' Seconds', 'wp-jpost'));
		printf('		t%s--; ', $div);
		printf('	}');
		printf('</script>');
	}

	/**
	 * 随机插入字符
	 * 
	 * @param txt 内容
	 * @param insert 关键字，可以是链接，数组
	 * 
	 * @since 2021.5.1
	 */
	private function jpost_randomInsert($txt, $insert)
	{
		//    if( !check_authoration() ){ return $txt;}
		/* 将内容拆分成数组，每个字符都是一个value，英文，中文，符号都算一个，只能在utf-8下中文才能拆分 */
		preg_match_all("/[\x01-\x7f]|[\xe0-\xef][\x80-\xbf]{2}/", $txt, $match);
		$delay = array();
		$add = 0;
		/* 获取不能插入的位置索引号($delay 数组)，也就是< > 之间的位置 */
		foreach ($match[0] as $k => $v) {
			if ($v == '<') $add = 1;
			if ($add == 1) $delay[] = $k;
			if ($v == '>') $add = 0;
		}
		$str_arr = $match[0];
		$len = count($str_arr);
		if (!is_array($insert)) {
			$insert_array = array($insert);
		} else {
			$insert_array = $insert;
		}
		foreach ($insert_array as $k => $v) {
			/* 获取随机插入的位置索引值 */
			$insertk = $this->jpost_insertK($len - 1, $delay);
			/* 循环将insert数据 拼接到 随机生成的索引 */
			$str_arr[$insertk] .= $insert[$k];
		}
		/* 合并插入 关键词后的数据，拼接成一段内容 */
		return join('', $str_arr);
	}

	/**
	 * 随机插入字符
	 * 
	 * @param count 随机索引值范围，也就是内容拆分成数组后的总长度-1
	 * 
	 * @param delay 不允许的随机索引值，也就是不能在 < > 之间
	 * 
	 * @since 2021.5.1
	 */
	private	function jpost_insertK($count, $delay)
	{
		$insertk = rand(0, $count);
		/* 索引值不能在 不允许的位置处（也就是< > 之内的索引值） */
		if (in_array($insertk, $delay)) {
			/* 递归调用，直到随机插入的索引值不在 <p > 这个索引值数组中 */
			$insertk = $this->jpost_insertK($count, $delay);
		}
		return $insertk;
	}

	/**
	 * 添加url地址参数
	 * 
	 * @param query_vars 允许的参数
	 * 
	 * @since 2021.5.1
	 */
	public function jpost_query_vars($query_vars)
	{
		array_push($query_vars, 'jpost');
		array_push($query_vars, 'jtask');
		array_push($query_vars, 'i');
		array_push($query_vars, 'jpage');
		array_push($query_vars, 'debug');
		return $query_vars;
	}

	/**
	 * 网络请求函数
	 * 如需要使用curl则使用snoopy请求网络
	 * 默认采用 WordPress 自带的网络函数
	 * 
	 * @param url 请求的网址
	 * @param args 参数
	 * @param errSKIP bool 跳过错误
	 * 
	 * @since 2021.5.1
	 */
	private function jpost_Handle($url, $args = array(), $accept_cache = false)
	{
		$cached_html = wp_cache_get($url);
		if($accept_cache && $cached_html){
			printf('<h1>正在是用缓存的HTML代码Using Cached Html Code: <code> %s </code></h1>',$url);
			return $cached_html;
		}
		$url = esc_url_raw(trim($url));
		/**
		 * 使用curl替换原生的WordPress HTTP API */
		$args = array(
			'user-agent'  => $this->handle_args['user-agent'],
			'timeout'     => 12,
			'headers'     => array(),
		);
		
		$response = wp_remote_get($url, $args);
		$this->html = wp_remote_retrieve_body($response);
		$httpCode = wp_remote_retrieve_response_code($response);
		$this->output_html .= $this->jpost_h1(__("请求的网址：", 'wp-jpost'));
		$this->output_html .= $this->jpost_output('<a rel="nofollow" target="_blank" href=' . esc_url($url) . '>' . esc_url($url) . '</a>');
		$this->output_html .= $this->jpost_h1(__("获取到的html内容（字符可能已被WordPress安全化过滤）", 'wp-jpost'));
		$this->output_html .= $this->jpost_output('<textarea style="width:100%;height:40%;">' . esc_html($this->jpost_char2UTF8($this->html)) . '</textarea>');
		if ($this->debug) {
			echo wp_kses($this->output_html, $this->JPost->allowed_html);
		}
		if ((is_wp_error($response) || 200 != $httpCode) && FALSE == $this->errSKIP) {
			// if (( curl_error($ch) || 200 != $httpCode) && FALSE == $errSKIP) {
			$this->output_html .= $this->jpost_h1(__("HTML状态码非200，程序已终止工作", 'wp-jpost'));
			wp_die($this->output_html);
		}
		wp_cache_set($url,$this->html);
		return $this->html;
	}

	/**
	 * 获取列表a链接
	 * 在后台“可视化XPath”选择标题、正文等内容时需要使用
	 * 
	 * @param url 网址
	 * @param ele 元素
	 * 
	 * @return array
	 * 
	 * @since 2021.5.22
	 */
	private function get_list_href($url, $ele)
	{
		$results_html = $this->jpost_Handle($url, $this->handle_args,true);
		$return_array = array();
		switch ($this->get_ele_method) {
			case 'xPath':
				$dom = new DOMDocument();
				libxml_use_internal_errors(TRUE);
				$dom->loadHTML($results_html);
				$xpath = new DOMXPath($dom);
				foreach ($ele as $item) {
					foreach ($xpath->query($item . '/@href') as $e) {
						$return_array[] = $e->nodeValue;
					}
				}
				return $return_array;
				break;
			case 'selector':
			default:
				/* 从列表页抓取单页a链接 */
				if (!function_exists('str_get_html')) {
					require_once  JPOST_DIR . "/includes/simple_html_dom.php";
				}
				$html = @str_get_html($results_html);
				if (!method_exists($html, 'find')) {
					wp_die(__('解析列表html异常，请稍候再试.', 'wp-jpost') . $url . '<textarea style="width:100%;height:40%;">' . esc_textarea($this->jpost_char2UTF8($results_html)) . '</textarea>');
				}
				$list_hrefs = @$html->find($ele[0]);
				foreach ($list_hrefs as $item) {
					$return_array[] = $item->href;
				}
				$html->clear();
				return $return_array;
				break;
		}
	}
	/**
	 * 使用XPath获取内容
	 * 
	 * @param string $xpath
	 * @return string[]
	 * 
	 * @since 2021.5.22
	 */
	function get_content_by_xpath($xpath, $inner = true)
	{
		// xPath object
		$xpathObj = new DOMXPath($this->dom);
		$xpathMatches = @$xpathObj->query("$xpath");
		$allMatchs = array();
		if ($xpathMatches == false) return $allMatchs;
		foreach ($xpathMatches as $element) {
			$matchHtml = ''; // single match html
			if ($inner) {
				$nodes = $element->childNodes;
				foreach ($nodes as $node) {
					$matchHtml .=  $this->dom->saveHTML($node) . "\n";
				}
			} else {
				$matchHtml =  $this->dom->saveHTML($element);
			}
			$allMatchs[] = $matchHtml;
		}
		return $allMatchs;
	}

	/**
	 * 获取单页的节点
	 * 
	 * @param ele 元素
	 * @param inner 是否包含html
	 * 
	 * @since 2021.5.22
	 */
	private function get_single_node($ele, $inner = false)
	{
		$ele = stripslashes($ele);
		switch ($this->get_ele_method) {
			case 'xPath':
				$this->dom  =  new DOMDocument;
				try {
					$internalErrors = libxml_use_internal_errors(true);
					@$this->dom->loadHTML($this->html);
					libxml_use_internal_errors($internalErrors);
				} catch (Exception $e) {
					throw new Exception('Failed to load the Document as a Dom');
				}
				$content = $this->get_content_by_xpath($ele, $inner);
				return $inner ? implode("\n", $content) : strip_tags($content[0]);
				break;
			case 'selector':
			default:
				$return_html = '';
				/* 从列表页抓取单页a链接 */
				if (!function_exists('str_get_html')) {
					require_once  JPOST_DIR . "/includes/simple_html_dom.php";
				}
				$html = @str_get_html($this->html);
				if (!method_exists($html, 'find')) {
					wp_die(__('解析html异常，请稍候再试.', 'wp-jpost') . $this->handle_url . '<textarea style="width:100%;height:40%;">' . esc_textarea($this->jpost_char2UTF8($this->html)) . '</textarea>');
				}
				foreach ($html->find($ele) as $item) {
					if ($inner) {
						$return_html .= '<p>' . $this->jpost_char2UTF8($item->innertext) . '</p>';
					} else {
						$return_html .= strip_tags($this->jpost_char2UTF8($item->innertext));
					}
				}
				$html->clear();
				return $return_html;
				break;
		}
	}

	/**
	 * 去除不必要的标签
	 * 
	 * @param content 内容
	 * @param unwanted_tags 不需要的标签
	 * 
	 * ！！！此函数，暂未完成！！！
	 * 
	 * @since 2021.5.22
	 */
	public function strip_unwanted($content, $unwanted_tags)
	{
		$unwanted_tags = is_array($unwanted_tags) ? array_filter($unwanted_tags) : $unwanted_tags;
		if (!$unwanted_tags) {
			return $content;
		}
		switch ($this->get_ele_method) {
			case 'xPath':
				// wp_die(__('暂不支持使用XPath去除元素','wp-jpost'));
				// break;
			case 'selector':
			default:
				if (!function_exists('str_get_html')) {
					require_once  JPOST_DIR . "/includes/simple_html_dom.php";
				}
				$html = @str_get_html($content);
				/* 删除指定元素并获取正文 */
				if (!method_exists($html, 'find')) {
					wp_die(__('解析html异常，请稍候再试.', 'wp-jpost'));
				}
				if (array_filter($unwanted_tags)) {
					foreach ($unwanted_tags as $tags_ele) {
						/* Delete all unwanted elements.删除所有不想要的元素 */
						foreach ($html->find($tags_ele) as $item) {
							$item->outertext = '';
						}
					}
				}
				$return_html = $html->innertext;
				$html->clear();
				return $return_html;
				break;
		}
	}

	/**
	 * 采集任务
	 * 
	 * @since 2021.5.1
	 */
	public function jpost_grab()
	{
		if (!is_singular('wp-jpost')) {
			return;
		}
		add_action('wp_before_admin_bar_render', array($this->JPost, 'admin_bar_render'));

		/** 初始化常量 */
		// $jpage = get_query_var('jpage') == 'auto' ? get_post_meta(get_the_ID(), 'cPage', true) : (int)get_query_var('jpage');
		if ('auto' == trim(get_query_var('jpage'))) {
			$jpage = (0 == get_post_meta(get_the_ID(), '_jpost_cPage', true)) ? 1 : get_post_meta(get_the_ID(), '_jpost_cPage', true);
		} else {
			$jpage = (int)get_query_var('jpage') > 0 ? (int)get_query_var('jpage') : (int)get_post_meta(get_the_ID(), '_jpost_cPage', true);
		}


		$this->debug = (int)get_query_var('debug');

		if ('auto' == trim(get_query_var('i'))) {
			$_jpost_i = get_post_meta(get_the_ID(), '_jpost_i', true);
			if ('' == $_jpost_i && $jpage > 1) {
				$jpage -= 1;
			}
		} else {
			$_jpost_i = get_query_var('i');
		}

		$theKey = $this->JPost->jpost_keys();
		foreach ($theKey as list($jkey, $jval)) {
			$$jkey = get_post_meta(get_the_ID(), $jkey, true);
		}
		if ($_jpost_task_get_ele_method) {
			$this->get_ele_method = $_jpost_task_get_ele_method;
		}

		if ($_jpost_check_login && !is_user_logged_in()) {
			wp_die(__('Access denied.'));
		}

		$this->_jpost_task_url = $_jpost_task_url;

		// include_once TEMPLATEPATH . '/header.php';
		wp_head();

		$this->errSKIP = $_jpost_skip_error;

		if ($_jpost_skip_error && $jpage) {
			/* 防止中断，自动刷新页面并添加随机参数 */
			$this->jpost_meta_go('', $_jpost_waiting_time);
		}

		/* 准备变量及检测环境开始 */
		if (isset($_jpost_single_page_elmt))
			$_jpost_single_page_elmt = str_replace(array('”', '“', '‘', '’'), array('"', '"', "'", "'"), $_jpost_single_page_elmt);

		/* 分类ID */
		$category = get_the_category();
		$post_category = array();
		if (is_array($category)) {
			foreach ($category as $v) {
				$post_category[] = $v->cat_ID;
			}
		}

		/* 分类标签 */
		$tags = get_the_tags();
		$tags_input = array();
		if (is_array($tags)) {
			foreach ($tags as $v) {
				$tags_input[] = $v->name;
			}
		}

		//  获取总页数
		$maxPage = $_jpost_task_list_max_page;

		/* 准备循环数据 */
		$url = '';
		$url = $_jpost_task_list_url;
		$cPage = $jpage >= 0 ? (int)$jpage : $maxPage;

		$jumpUrl = get_the_permalink() . '?' . (isset($jtask) ? 'jtask=' . $jtask : '') . '&jpage={page}';
		// if( $_jpost_random_in_url )	
		// 	$jumpUrl .= '&random='.wp_generate_password('12',false);
		$url = str_replace('{page}', $cPage, $url);
		$jumpUrl .= $this->debug ? '&debug=1' : '';
		// $post_category = $category;

		/* 准备采集页面板菜单 */
		$this->output_html = $this->jpost_output('') . $this->JPost->jpost_ads() . $this->jpost_output('');
		$enter = '<br />';

		$results = '';

		/* 构造请求参数 */
		$args = array();
		if ($_jpost_ua) {
			// $args['agent'] = $_jpost_ua;   
			$args['user-agent'] = $_jpost_ua;
		} else {
			$args['user-agent'] = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/90.0.4430.212 Safari/537.36 Edg/90.0.818.66';
		}
		// if( isset($_jpost_curl) && $_jpost_curl )       			
		// 	$args['curl'] = $_jpost_curl;
		// if( $_jpost_cookies )    			
		// 	$args['cookies'] = $_jpost_cookies;
		// if( strpos($url, 'https://') !== false )   
		// 	$args['port'] = 443;
		// if( isset( $_jpost_proxy_host ) ) 	
		// 	$args['proxy_host'] = $_jpost_proxy_host;
		// if( isset( $_jpost_proxy_port ) ) 	
		// 	$args['proxy_port'] = $_jpost_proxy_port;
		// if( isset( $_jpost_proxy_user ) ) 	
		// 	$args['proxy_user'] = $_jpost_proxy_user;
		// if( isset( $_jpost_proxy_pass ) )	
		// 	$args['proxy_pass'] = $_jpost_proxy_pass;

		if (array_filter($args)) {
			$this->handle_args = $args;
		}

		/* 采集结束,抛出显示 */
		if ($cPage <= 0) {
			$this->JPost->meta_box();
			$this->output_html .= $this->jpost_h1('<span style="color:green;">' . __(":) 任务未开始或已完成.", 'wp-jpost') . '</span>');
			echo wp_kses($this->output_html, $this->JPost->allowed_html);
			// include_once TEMPLATEPATH . '/footer.php';
			wp_footer();
			exit;
		}

		/* 获取列表数据 */
		$list_hrefs = $this->get_list_href($url, $_jpost_task_list_a_element);
		/* 抓取不到文章链接 */
		// if( !is_array( $list_hrefs ) || empty( $list_hrefs ) ) { $this->output_html .=  __("网址错误：",'wp-jpost').$url.$enter; }

		/* 控制页面元素 */
		if ('' == $_jpost_i) {
			$_jpost_i = count($list_hrefs) - 1;
		} elseif ($_jpost_i >= 0) {
			$_jpost_i = (int)$_jpost_i;
		} else {
			wp_die('Index Error.');
		}

		/* 文章列表页面内容 */
		$single_url = '';
		$single_url = $list_hrefs[$_jpost_i];
		if (!$single_url) {
			$this->output_html .= $this->jpost_h1(__('网址', 'wp-jpost'));
			$this->output_html .= $this->jpost_output('<a rel=”nofollow” href="' . $url . '"> ' . $url . ' </a>');
			$this->output_html .= $this->jpost_h1(__('返回HTML', 'wp-jpost'));
			$this->output_html .= $this->jpost_output('<textarea style="width:100%;height:40%;">' . esc_textarea($this->html) . '</textarea>');
			$this->output_html .= $this->jpost_output(__("获取列表网址href属性错误", 'wp-jpost'));
			echo wp_kses($this->output_html, $this->JPost->allowed_html);
			if ($_jpost_skip_error) {
				$JPost->jpost_meta_go($JPost->getNextURL($_jpost_i, $cPage, $jumpUrl), (int)$_jpost_waiting_time);
			}
			wp_die('', 'WP-JPOST');
		}
		if (strpos($single_url, "http://") === false && strpos($single_url, 'https://') === false) {
			$single_url = $_jpost_task_url . $single_url;
		}
		$single_url = str_replace('../', '', $single_url);
		$outputUrl = $single_url;

		if ($this->debug) {
			$this->output_html .=  $this->jpost_h1(__('列表', 'wp-jpost'));
			foreach ($list_hrefs as $single_href) {
				$this->output_html .=  $this->jpost_output($single_href);
			}
		}

		/* 单页请求开始 */
		$this->handle_url = $single_url;
		$output_html = $this->output_html;
		$this->output_html = '';
		$this->html = $this->jpost_Handle($this->handle_url, $this->handle_args);
		$this->output_html = $output_html . $this->output_html;

		/* 获取标题 */
		$post_title = $this->get_single_node($_jpost_single_page_title, false);

		$post_content = $this->get_single_node($_jpost_single_page_content, true);

		/* 删除指定元素并获取正文 */
		$post_content = $this->strip_unwanted($post_content, $_jpost_remove_tags);

		if (is_array($_jpost_search) && array_filter($_jpost_search)) {
			/* 替换内容 */
			$post_title = str_ireplace($_jpost_search, $_jpost_replace, $post_title);
			$post_content = str_ireplace($_jpost_search, $_jpost_replace, $post_content);
		}

		if (is_array($_jpost_random_keyword) && array_filter($_jpost_random_keyword)) {
			/* 随机插入关键词 */
			$rand_keys = array_rand($_jpost_random_keyword);
			$post_title .= '_' . $_jpost_random_keyword[$rand_keys];

			$rand_keys = array_rand($_jpost_random_keyword);
			$post_content = $this->jpost_char2UTF8($this->jpost_randomInsert($post_content, $_jpost_random_keyword[$rand_keys]));
		}

		$next = array();
		$next = $this->getNextURL($_jpost_i, $cPage, $jumpUrl);

		/* 设置页配置获取 */
		if ($post_title && $post_content) {
			/* 检查内容 */
			$the_query = '';
			$post_ID = '';
			$post_name = '';

			$args =  array(
				'meta_query' => array(
					array(
						'key' 	=> $this->JPost->source,
						'value' => $single_url,
					),
				),
			);

			query_posts($args);
			if (have_posts()) {
				/* 修改文章 */
				$status = __('修改', 'wp-jpost');
				while (have_posts()) {
					the_post();
					$post_ID = get_the_ID();
					if ($_jpost_coverage_mode) {
						$my_post = array(
							'ID'            =>  $post_ID,
							'post_content'  =>  $post_content,
							"Auto_Remove_Link"  => $_jpost_autoremove_links,
						);
						$my_post['post_content'] .= '<!--This article posted by WP-JPost plugin, URL : http://www.jiloc.com/go/wp-jpost  -->';

						/* 停止过滤字符 */
						kses_remove_filters();
						wp_update_post($my_post);
						kses_init_filters();
					} else {
						$status = __('跳过', 'wp-jpost');
						// printf('<h1><a href="%s">%s</a>-->%s</h1>', get_the_permalink(), get_the_title(), get_post_meta(get_the_ID(), $this->JPost->source, true));die;
						echo $this->jpost_output(__('检测到相同内容：跳过', 'wp-jpost'));
					}
				}
				/* 重置数据 */
				wp_reset_postdata();
			} else {
				/* Wordpress 入库 */
				$status = __('已插入', 'wp-jpost');
				$my_post = array(
					// "Auto_Remove_Link"  =>  $_jpost_autoremove_links,
					"post_title"    	=>  $post_title,
					"post_content"  	=>  $post_content,
					'post_status'   	=>  $_jpost_post_status ? $_jpost_post_status : 'publish',
					'post_author'   	=>  get_post_field('post_author', get_the_ID()),
					'post_category' 	=>  $post_category,
					'tags_input'    	=>  $tags_input,
					'post_name'     	=>  $post_name ? $post_name : $post_title,
					"post_date"     	=>  date("Y-m-d H:i:s", time()),
					'guid'          	=>  '',
					// 'jpost_curl'        =>  $_jpost_curl,
				);

				if ($_jpost_autoremove_links) {
					$my_post['post_content'] = stripslashes($my_post['post_content']);
					$my_post['post_content'] = preg_replace("#<a[^>]*>(.*?)</a>#is", "$1", $my_post['post_content']);
					$my_post['post_content'] = AddSlashes($my_post['post_content']);
				}
				$my_post['post_content'] .= '<!--This article posted by WP-JPost plugin, URL : http://www.jiloc.com/go/wp-jpost  -->';
				/* 停止过滤字符 */
				kses_remove_filters();
				$post_ID = wp_insert_post($my_post);
				kses_init_filters();
			}

			if ($this->debug) {
				$my_post = get_post($post_ID);
				$this->output_html .=  $this->jpost_h1(__('标题', 'wp-jpost'));
				$this->output_html .=  $my_post->post_title . $enter;
				$this->output_html .= $this->JPost->jpost_ads();
				$this->output_html .=  $this->jpost_h1(__('内容', 'wp-jpost'));
				$this->output_html .=  $my_post->post_content . $enter;
				wp_delete_post($post_ID);
				$this->output_html .=  $this->jpost_output(__("该文章已删除.", 'wp-jpost'));
				$this->output_html .=  $this->jpost_h1('<span style="color:#28a929">(~‾⌣‾)~ ' . __('恭喜! 调试成功，您现在可以采集了!', 'wp-jpost') . '</span>');
			} else {
				$insertUrl = get_permalink($post_ID);
				$this->output_html .=  $this->jpost_output("{$outputUrl} => {$status} <a href='{$insertUrl}' target='_blank'>{$insertUrl}</a>");
				$this->output_html .=  $this->jpost_h1('<span style="color:#28a929">(^_^) ' . __('工作中... , 请勿关闭本页面！', 'wp-jpost') . '</span>');
			}
			unset($downloads);
			unset($the_query);
			unset($singleSnoopy);
			$this->output_html .=  $enter;
		} else {
			$this->output_html .= $this->jpost_h1(__('当前网址', 'wp-jpost'));
			$this->output_html .= $this->jpost_output('<a target="_blank" href="' . $single_url . '">' . $single_url . '</a>');
			$this->output_html .= $this->jpost_h1(__('标题', 'wp-jpost'));
			$this->output_html .= $this->jpost_output('<textarea style="width:100%;height:40%;">' . $post_title . '</textarea>');
			$this->output_html .= $this->jpost_h1(__('正文', 'wp-jpost'));
			$this->output_html .= $this->jpost_output('<textarea style="width:100%;height:40%;">' . $post_content . '</textarea>');
			$this->output_html .= $this->jpost_h1(__('HTML', 'wp-jpost'));
			$this->output_html .= $this->jpost_output('<textarea style="width:100%;height:40%;">' . $results . '</textarea>');
			$this->output_html .= $this->jpost_output("<span style='color:red'>꒪̆౪̮꒪ " . __('标题/正文内容 空NULL , 已为您跳过', 'wp-jpost') . "</span>");
			echo wp_kses($this->output_html, $this->JPost->allowed_html);
			if ($_jpost_skip_error) {
				$this->jpost_meta_go($next['url'], (int)$_jpost_waiting_time);
			}
			return;
		}

		update_post_meta($post_ID, $this->JPost->source, $single_url);

		$next_array = $next['array'];

		update_post_meta(get_the_ID(), '_jpost_cPage', $next_array['cPage']);

		if ($next_array['i'] > 0) {
			update_post_meta(get_the_ID(), '_jpost_i', $next_array['i']);
		} else {
			delete_post_meta(get_the_ID(), '_jpost_i');
		}

		echo wp_kses($this->output_html, $this->JPost->allowed_html);
		$this->jpost_meta_go($next['url'], (int)$_jpost_waiting_time);
		echo $next['html'];
		// include(TEMPLATEPATH . '/footer.php');
		wp_footer();
		exit;
	}

	/**
	 * 构造写一个跳转的url地址
	 * 
	 * @since 2021.5.1
	 */
	private function getNextURL($_jpost_i, $cPage, $jumpUrl, $return_array = false)
	{
		/* 当前页面最后一个元素，页面ID-- */
		if ($_jpost_i <= 0) {
			$cPage--;
			$jumpUrl = str_replace('{page}', $cPage, $jumpUrl);
		} else {
			$jumpUrl .= '&i={i}';
			$_jpost_i--;
			$jumpUrl = str_replace(array('{page}', '{i}'), array($cPage, $_jpost_i), $jumpUrl);
		}
		$array =  array('i' => $_jpost_i, 'cPage' => $cPage);
		$jumpUrl = esc_url_raw($this->JPost->replaceSymbolURL($jumpUrl));
		$html = $this->jpost_output(__("下一个网址", 'wp-jpost') . "： <a href='{$jumpUrl}'>" . $jumpUrl . "</a>");
		echo $html;
		return array(
			'array' 	=>	$array,
			'html'		=>	$html,
			'url'		=>	$jumpUrl,
		);
	}

	/**
	 * 定位功能
	 * 
	 * @return xpath
	 * 
	 * @since 2021.5.24
	 */
	public function iframe_callback()
	{
		//auth check	
		if (!current_user_can('administrator')) die();

		// Detect the URL
		$_GET['url'] = esc_url_raw($_GET['address']);
		$url = null;

		//detect cookie
		$cookie = isset($_GET['theCookie']) ? sanitize_text_field($_GET['theCookie']) : '';

		$url = esc_url_raw($_GET['url']);

		// about:blank if passed no URL
		if (!$url) {
			header('Location: about:blank');
			exit();
		}
		$type = sanitize_text_field($_GET['type']);
		if ($type == 'single') {
			$list_a_first = sanitize_text_field(urldecode($_GET['list_a_first']));
		}

		$args = array(
			'method' 	=> 'GET',
			'cookie'	=>	$cookie,
		);

		$content = $this->jpost_Handle($url, $args);

		// Supply base element to resolve relative path
		if (!preg_match('/<base\s/i', $content)) {
			$base = '<base href="' . $url . '">';
			$content = preg_replace('{(<head.*?>)}',  "$1$base", $content);
		}

		//fix href="//
		$content = str_replace('src="//', 'src="https://', $content);
		$content = str_replace('href="//', 'href="https://', $content);

		// Supply protocol and domain before absolute path turns 'href="/' to 'href="http:domain.com/'
		if (preg_match('!^https?://[^/]+!', $url, $matches)) {
			$stem = $matches[0];
			$content = preg_replace('!(\s)(src|href)(=")/!i', "\\1\\2\\3$stem/", $content);
			$content = preg_replace('!(\s)(url)(\s*\(\s*["\']?)/!i', "\\1\\2\\3$stem/", $content);
		}

		if ($type == 'single' && $list_a_first != '') {
			/* 从列表页抓取单页a链接 */
			if (!function_exists('str_get_html')) {
				require_once  JPOST_DIR . "/includes/simple_html_dom.php";
			}
			$this->get_ele_method = 'xPath';
			$this->handle_url = $url;
			$single_url = $this->get_single_node(stripslashes($list_a_first) . '/@href', 1);
			$single_url = is_array($single_url) ? $single_url[0] : $single_url;
			if (strpos($single_url, "http://") === false && strpos($single_url, 'https://') === false) {
				$single_url = sanitize_text_field(stripslashes($_GET['baseurl']) . $single_url);
			}
			$content = $this->jpost_Handle($single_url, $this->handle_args);
		}

		//strip scripts $res['cont'] = preg_replace('{<script.*?script>}s', '', $res['cont']);
		$beforeJS = $content;
		$content = preg_replace('{<script.*?</script>}s', '', $content);

		if (trim($content) == '') $content = $beforeJS; //sometimes replace returns NULL ticket #7848


		echo $content . "<style>
							body {
								font-family: sans-serif;
							}
							.highlight {
								box-shadow:inset 0 0 0 1000px rgba(255, 0, 0, 0.5) !important;
								outline: 1px solid red !important;
							}
						</style>";
		die();
	}
}
