<?php
if (!defined('ABSPATH')) {
	exit;
}
/**
 * 采集配置及WP后台显示
 * 
 * @since 2019.10.1
 */
class JPost
{

	protected $jpost_readme = 'https://www.jiloc.com/go/wp-jpost';
	protected $post_type = 'wp-jpost';
	protected $rewrite_slug = 'jpost';

	public $allowed_html = array(
		'a'      => array(
			'href'  => array(),
			'title' => array(),
		),
		'br'     	=> 	array(),
		'em'     	=> 	array(),
		'strong' 	=> 	array(),
		'span'		=> 	array(),
		'img'		=>	array(),
		'p'			=>	array(),
		'div'		=>	array(),
		'textarea'	=>	array(),
		'script'	=>	array(),
		'img'		=>	array(),
	);

	/** 文章来源字段 */
	public $source = '_jpost_source';

	/**
	 * Constructor.
	 */
	public function __construct()
	{
		add_action('plugins_loaded', array($this, 'load_textdomain'));
		add_action('init', array($this, 'register_post_type'));
		// add_filter( 'post_type_link', array($this , 'custom_link' ), 1, 3);
		add_filter('post_updated_messages', array($this, 'updated_message'));
		// add_action( 'admin_init',array($this,'register_setting_page'));
		add_action('admin_menu', array($this, 'add_meta_box'));
		add_action('save_post', array($this, 'meta_box_save'), 1, 2);
		add_action('manage_wp-jpost_posts_custom_column', array($this, 'jpost_columns_data'));
		add_filter('manage_wp-jpost_posts_columns', array($this, 'jpost_columns_filter'));
		add_filter('manage_edit-post_columns', array($this, 'columns_filter'));
		add_action('manage_posts_custom_column', array($this, 'columns_data'));
		add_action('admin_notices', array($this, 'jpost_notice'));
		add_action('admin_enqueue_scripts', array($this, 'add_admin_script'));
		add_action('init', [$this, 'filter_post_rewrite_rules'], 11);
		add_action('wp_before_admin_bar_render', array($this, 'jpost_admin_bar'));
	}

	/**
	 * 列表自定义链接
	 * 
	 * @since 2021.5.1
	 */
	public function custom_link($permalink, $post)
	{
		if ($post->post_type == $this->post_type) {
			return $permalink . (strpos($permalink, '?') !== false ? '&' : '?') . 'jpage=' . get_post_meta(get_the_ID(), '_jpost_task_list_max_page', true) . '&debug=1';
		} else {
			return $permalink;
		}
	}

	/**
	 * 第一次安装提醒用户设置“固定链接”
	 * 
	 * @since 2021.5.1
	 * 
	 * VERSION 1.2 中已弃用
	 */
	public function jpost_notice()
	{
		if (get_post_type() == $this->post_type)
			echo wp_kses($this->jpost_ads(), $this->allowed_html);
		/* Check transient, if available display notice */
		if (get_transient('jpost-notice')) {
			printf('<div class="notice notice-warning is-dismissible">');
			printf('<p>%s</p>', __('WP-JPost 需要您到 [ <strong>仪表盘</strong> ] -> [ <strong>设置</strong> ] -> [ <strong>固定链接</strong> ] -> 点击 [ <strong>保存</strong> ] 按钮. [ <a href="https://www.jiloc.com/44235.html">?</a> ]', 'wp-jpost'));
			printf('</div>');
			/* Delete transient, only display this notice once. */
			delete_transient('jpost-notice');
		}
	}

	/**
	 * 重写自定义文章类型伪静态
	 * 
	 * @since 2021.5.19
	 **/
	public function filter_post_rewrite_rules()
	{
		flush_rewrite_rules();
	}

	/**
	 * 采集任务配置字段
	 * 
	 * @since 2021.5.1
	 */
	public function jpost_keys()
	{
		return array(
			array('_jpost_task_url', __('根目录网址', 'wp-jpost'), 'https://www.jiloc.com'),
			array('_jpost_task_list_url', __('列表页网址', 'wp-jpost'), 'https://www.jiloc.com/page/{page}'),
			array('_jpost_task_list_max_page', __('最大分页', 'wp-jpost'), '1'),
			array(
				'_jpost_task_get_ele_method', __('获取元素方法', 'wp-jpost'),
				array(
					'selector' => __('JQUERY/CSS选择器'),
					// 'xPath' => __('XPath可视化', 'wp-jpost')
				)
			),
			array('_jpost_task_list_a_element', __('列表中a元素', 'wp-jpost'), 'h1.entry-title a'),
			array('_jpost_single_page_title', __('单页标题', 'wp-jpost'), 'h1.entry-title'),
			array('_jpost_single_page_content', __('单页正文', 'wp-jpost'), 'div.entry-content'),
			array('_jpost_post_status', __('发布状态', 'wp-jpost'), 'publish'),
			array('_jpost_skip_error', __('跳过错误', 'wp-jpost'), '1'),
			array('_jpost_check_login', __('必须登陆才能采集', 'wp-jpost'), '1'),
			// array( '_jpost_random_in_url' , __( 'Random in URL' , 'wp-jpost' ) ,'1'),
			array('_jpost_waiting_time', __('等待时间', 'wp-jpost'), '6'),
			// array( '_jpost_auto_save_images' , __( '启用Auto Save Images插件' , 'wp-jpost' ) ,''),
			array('_jpost_autoremove_links', __('去除文章内容的a链接', 'wp-jpost'), ''),
			array('_jpost_remove_tags', __('去除selector元素', 'wp-jpost'), ''),
			array('_jpost_search', esc_html__('搜索文字', 'wp-jpost'), ''),
			array('_jpost_replace', esc_html__('替换文字', 'wp-jpost'), ''),
			array('_jpost_random_keyword', __('随机关键词', 'wp-jpost'), ''),
			array('_jpost_coverage_mode', __('重复时覆盖', 'wp-jpost'), ''),
			// array( '_jpost_baidu_translate' , __( 'Use Baidu Translate' , 'wp-jpost' ) ,''),
			// array( '_jpost_curl' , __( 'Curl路径' , 'wp-jpost' ) ,''),
			array('_jpost_ua', __('User Agent', 'wp-jpost'), ''),
			// array( '_jpost_cookies' , __( 'Cookies' , 'wp-jpost' ) ,''),
			// array( '_jpost_proxy_host' , __( '代理主机:端口' , 'wp-jpost' ) ,''),
			// array( '_jpost_proxy_user' , __( '代理用户名' , 'wp-jpost' ) ,''),
			// array( '_jpost_proxy_passwd' , __( '代理密码' , 'wp-jpost' ) ,''),
			// array( '_jpost_source' , __( '来源' , 'wp-jpost' ) ,''),
			// '_jpost_login',
			// '_jpost_loginurl',
			// '_jpost_params',
			// '_jpost_single_page_other_elmt',
			// '_jpost_single_page_other_fetch',
		);
	}

	/**
	 * 文章列表添加"来源".
	 *
	 * @param  array $column Columns.
	 * 
	 * @return array
	 * 
	 * @since 2021.5.24
	 */
	public function columns_filter($columns)
	{
		return array_merge($columns, array(
			$this->source => __('来源', 'wp-jpost'),
			// 'created_time'	=>	__('创建时间/采集时间','wp-jpost'),
		));
	}

	/**
	 * 文章列表添加"来源".
	 *
	 * @param  array $column Columns.
	 * 
	 * @since 2021.5.24
	 */
	public function columns_data($column)
	{
		$allowed_tags = array(
			'a' => array(
				'href' => array(),
				'rel'  => array(),
			),
		);

		if ($column ==  $this->source) {
			// $source = wp_kses( make_clickable( esc_url( get_post_meta( get_the_ID() , $this->source ,true ) ) ), $allowed_tags );
			$source = get_post_meta(get_the_ID(), $this->source, true);
			echo wp_kses($source ? $source : '<span aria-hidden="true">—</span>', $this->allowed_html);
		}

		// if ( $column ==  'created_time' ) {
		// 	the_time('Y/m/d G:i:s');
		// }

	}

	/**
	 * Colum filter.
	 * 
	 * @since 2021.5.1
	 * 
	 * @param  array $columns Columns.
	 * 
	 * @return array Filtered columns.
	 */
	public function jpost_columns_filter($columns)
	{
		$columns = array(
			'cb'        => '<input type="checkbox" />',
			'title'     => __('任务名'),
			'url'       => __('网址', 'wp-jpost'),
			'cron'		=> __('定时', 'wp-jpost') . '[ <a href="' . $this->jpost_readme . '">?</a> ]',
			'process' => __('进度', 'wp-jpost'),
			'options'	=> __('操作', 'wp-jpost'),
		);
		return $columns;
	}

	/**
	 * 自定义类型列表.
	 *
	 * @param  array $column Columns.
	 */
	public function jpost_columns_data($column)
	{
		$title = get_the_title();
		$jpage = get_post_meta(get_the_ID(), '_jpost_task_list_max_page', true);
		$curtten_page = get_post_meta(get_the_ID(), '_jpost_cPage', true);
		$url   = str_replace('{page}', $curtten_page, get_post_meta(get_the_ID(), '_jpost_task_list_url', true));
		$i = get_post_meta(get_the_ID(), '_jpost_i', true);
		switch ($column) {
			case 'title':
				echo wp_kses($title, $this->allowed_html);
				break;
			case 'url':
				echo wp_kses(make_clickable(esc_url($url ? $url : '')), $this->allowed_html);
				break;
			case "cron":
				$jpage = $i = 'auto';
				echo wp_kses(make_clickable($this->replaceSymbolURL(get_the_permalink() . '?&' . http_build_query(array('jpage' => $jpage, 'i' => $i,)))), $this->allowed_html);
				break;
			case 'process':
				echo wp_kses('<p>' . __('页码：', 'wp-jpost') . $curtten_page . ' / ' . $jpage . '</p>', $this->allowed_html);
				echo wp_kses('<p>' . __('页面`a`序号：', 'wp-jpost') . (int)$i . '</p>', $this->allowed_html);
				break;
			case 'debug':
				echo wp_kses(make_clickable(get_the_permalink()), $this->allowed_html);
				break;
			case 'options':
				printf('[<a style="margin:12px;" href="%s">%s</a>]', $this->constructURL('debug'), __('调试', 'wp-jpost'));
				printf('[<a style="margin:12px;" href="%s">%s</a>]', $this->constructURL('continue'), __('继续', 'wp-jpost'));
				printf('[<a style="margin:12px;" href="%s">%s</a>]', $this->constructURL('restart'), __('重新开始', 'wp-jpost'));
			default:
				break;
		}
		if (get_transient('jpost-notice'))
			set_transient('jpost-notice', true);
	}

	/**
	 * 更新信息.
	 *
	 * @param  array $messages Messages.
	 *
	 * @return array           Messages.
	 */
	public function updated_message($messages)
	{
		$jpost_object = get_post_type_object($this->post_type);
		$messages[$this->post_type] = $jpost_object->labels->messages;
		// $permalink = get_permalink();
		$permalink = $this->constructURL('debug');
		if ($permalink) {
			foreach ($messages[$this->post_type] as $id => $message) {
				$messages[$this->post_type][$id] = sprintf($message, $permalink);
			}
		}
		return $messages;
	}

	/**
	 * 添加 metabox.
	 * 
	 * @since 2021.5.1
	 */
	public function add_meta_box()
	{
		add_menu_page('wp_jpost', 'JPost采集', 'administrator', 'wp-jpost', '', '', 100);
		add_submenu_page('wp-jpost', __('添加', 'wp-jpost'), __('添加', 'wp-jpost'), 'administrator', 'post-new.php?post_type=wp-jpost', '');
		add_meta_box('jpost', __('任务信息', 'wp-jpost'), array($this, 'meta_box'), 'wp-jpost', 'normal', 'high');
		// add_meta_box( 'jpost-others', __( '其他信息','wp-jpost' ), array( $this, 'meta_box_others' ), 'wp-jpost', 'normal', 'high' );	
	}

	/**
	 * 加载后台JS
	 *
	 * @since 1.0.0
	 * @param string $hook
	 */
	public function add_admin_script($hook)
	{
		if ($this->post_type == get_post_type() && 'post.php' === $hook || 'post-new.php' === $hook) {
			wp_register_style('wp-jpost', JPOST_URL . '/assets/css/plugin.css');
			wp_enqueue_style('wp-jpost');
			// wp_register_script('cssify',  JPOST_URL.'/assets/js/cssify.js', array('jquery'),JPOST_VERSION, true);
			// wp_enqueue_script('cssify');			
			wp_register_script('jpost',  JPOST_URL . '/assets/js/plugin.js', array('jquery'), '', true);
			wp_enqueue_script('jpost');
			$dataToJS = array(
				'url'   => JPOST_URL . '/assets/',
			);
			wp_localize_script('jpost', 'assets', $dataToJS);
		}
	}

	/**
	 * Metabox 显示
	 * 
	 * @since 2021.5.1
	 */
	public function meta_box()
	{
		printf('<input type="hidden" name="_jpost_nonce" value="%s" />', esc_attr(wp_create_nonce(plugin_basename(__FILE__))));
		$key = $this->jpost_keys();
		echo wp_kses($this->jpost_ads(), $this->allowed_html);
		$required = array(
			'_jpost_task_url',
			'_jpost_task_list_url',
			'_jpost_task_list_a_element',
			'_jpost_single_page_title',
			'_jpost_single_page_content'
		);
		$select_format = array('_jpost_task_get_ele_method',);
		$cron_url = $this->replaceSymbolURL(get_the_permalink() . '?&' . http_build_query(array('jpage' => 'auto', 'i' => 'auto')));
		foreach ($key as list($k, $v, $e)) {
			$kv = get_post_meta(get_the_ID(), $k, true);
			switch ($k) {
				case '_jpost_search':
					$replaceAry = get_post_meta(get_the_ID(), '_jpost_replace', true);
					if (is_array($kv)) {
						foreach ($kv as $kk => $vv) {
							printf('<div class="jpost-item"><div class="item-box">');
							printf('<div class="label">%s</div>', $v);
							printf('<div class="ele %s"><textarea name="%s[]" data-tname="%s" data-rname="%s">%s</textarea></div>', $k, $k, __('搜索', 'wp-jpost'), __('替换', 'wp-jpost'), $vv);
							printf('<div class="label label%s">%s</div>', '_jpost_replace', __('替换', 'wp-jpost'));
							printf('<div class="ele %s"><textarea name="%s[]" data-rname="%s">%s</textarea></div>', '_jpost_replace', '_jpost_replace', __('替换', 'wp-jpost'), $replaceAry[$kk]);
							if ($kk == 0) {
								printf('<div class="add_search_replace"><input type="button" name="add_search_replace" class="add_search_replace" value=" + " /></div>');
							} else {
								printf('<div class="del_search_replace"><input type="button" name="del_search_replace" class="del_search_replace" value=" - "></div>');
							}
							printf('</div></div>');
						}
					} else {
						printf('<div class="jpost-item"><div class="item-box">');
						printf('<div class="label">%s</div>', $v);
						printf('<div class="ele %s"><textarea name="%s[]" data-tname="%s" data-rname="%s">%s</textarea></div>', '_jpost_search', '_jpost_search', __('搜索', 'wp-jpost'), __('替换', 'wp-jpost'), '');
						printf('<div class="label label%s">%s</div>', '_jpost_replace', __('替换', 'wp-jpost'));
						printf('<div class="ele %s"><textarea name="%s[]">%s</textarea></div>', '_jpost_replace', '_jpost_replace', '');
						printf('<div class="add_search_replace"><input type="button" name="add_search_replace" class="add_search_replace" value=" + " /></div>');
						printf('</div></div>');
					}
					break;
				case '_jpost_replace':
					break;
				case '_jpost_post_status':
					$statuses = array_reverse(get_post_statuses());
					$jstatus = $kv ? $kv : 'publish';
					printf('<div class="jpost-item">');
					printf('<div class=label>%s</div>', $v);
					printf('<div class="ele %s">', $k);
					printf('<select name="%s" id="%s" value="%s">', $k, $k, $jstatus);
					foreach ($statuses as $i => $j) {
						printf('<option value="%s" %s>%s</option>', $i, $i === $jstatus ? "selected" : '', $j);
					}
					printf('</select> </div></div>');
					break;
				case in_array($k, array(
					'_jpost_auto_save_images',
					'_jpost_autoremove_links',
					'_jpost_coverage_mode',
					'_jpost_baidu_translate',
					'_jpost_skip_error',
					'_jpost_check_login',
					'_jpost_random_in_url'
				)):
					printf('<div class="jpost-item">');
					printf('<div class=label>%s</div>', $v);
					printf('<div class="ele %s"><input type="checkbox" type="text" name="%s" %s></div>', $k, $k, $kv ? 'checked="checked"' : '');
					printf('</div>');
					break;
				case in_array($k, array('_jpost_remove_tags', '_jpost_random_keyword')):
					printf('<div class="jpost-item">');
					if (is_array($kv)) {
						foreach ($kv as $kk => $vv) {
							printf('<div class="item-box">');
							printf('<div class="label">%s</div>', $v);
							printf('<div class="ele %s"><textarea name="%s[]" data-name="%s" data-tname="%s" placeholder="%s" >%s</textarea></div>', $k, $k, $k, $v, $vv, $vv ? $vv : $e);
							if ($kk == 0) {
								printf('<div class="add_tags"><input type="button" name="add_tags" class="add_tags" value=" + " /></div>');
							} else {
								printf('<div class="del_tags"><input type="button" name="del_tags" class="del_tags" value=" - "></div>');
							}
							printf('</div>');
						}
					} else {
						printf('<div class="item-box">');
						printf('<div class="label">%s</div>', $v);
						printf('<div class="ele %s"><textarea name="%s[]" data-name="%s" data-tname="%s">%s</textarea></div>', $k, $k, $k, $v, '');
						printf('<div class="add_tags"><input type="button" name="add_tags" class="%s" value=" + " /></div>', $k, $k);
						printf('</div>');
					}
					printf('</div>');
					break;
				case in_array($k, $required):
					printf('<div class="jpost-item">');
					printf('<div class=label>%s [<span style="color:red;"> * </span>] </div>', $v);
					if (in_array($k, ['_jpost_task_list_a_element'])) {
						if (!is_array($kv)) {/* selector界面 */
							printf(
								'<div class="item-box">
										<div class="ele %s"><input type="text" name="%s[]" value="%s" placeholder="%s" %s style="width:80%%;" data-name="%s" data-tname="%s" />
											<button title="%s" data-popup-field-name class="visual_selector">%s</button>
										</div>
										<div class="add_tags"><input type="button" name="add_a_tags" class="%s" value=" + " /></div>
									</div>',
								$k,
								$k,
								esc_attr($kv),
								$e,
								'required',
								$k,
								$k,
								__('可视化定位', 'wp-jpost'),
								__('可视化定位', 'wp-jpost'),
								$k
							);
						} else {	/* 可视化界面 */
							foreach ($kv as $kk => $vv) {
								printf(
									'<div class="item-box">
											<div class="ele %s"><input type="text" name="%s[]" value="%s" placeholder="%s" %s style="width:80%%;" data-name="%s" data-tname="%s" />
												<button title="%s" data-popup-field-name class="visual_selector">%s</button>
											</div>',
									$k,
									$k,
									esc_attr($vv),
									$e,
									'required',
									$k,
									$k,
									__('可视化定位', 'wp-jpost'),
									__('可视化定位', 'wp-jpost')
								);
								if ($kk == 0) {
									printf('<div class="add_tags"><input type="button" name="add_a_tags" class="%s" value=" + " /></div>', $k);
								} else {
									printf('<div class="del_tags"><input type="button" name="del_tags" class="del_tags" value=" - "></div>');
								}
								printf('</div>');
							}
						}
					} elseif (in_array($k, ['_jpost_single_page_title', '_jpost_single_page_content'])) {
						printf(
							'<div class="ele %s"><input type="text" name="%s" value="%s" placeholder="%s" %s style="width:80%%;"/>
									<button title="%s" data-popup-field-name class="visual_selector">%s</button>
								</div>',
							$k,
							$k,
							esc_attr($kv),
							$e,
							'required',
							__('可视化定位', 'wp-jpost'),
							__('可视化定位', 'wp-jpost')
						);
					} else {
						printf('<div class="ele %s"><input type="text" name="%s" value="%s" placeholder="%s" %s /></div>', $k, $k, esc_attr($kv), $e, 'required');
					}
					printf('</div>');
					break;
				case in_array($k, $select_format):
					printf('<div class="jpost-item">');
					printf('<div class=label>%s [<span style="color:red;"> * </span>] </div>', $v);
					printf('<div class="ele %s"><select name="%s">', $k, $k);
					foreach ($e as $select_val => $select_name) {
						printf('<option value="%s" %s>%s</option>', $select_val, $select_val == $kv ? 'selected' : '', $select_name);
					}
					printf('</select></div>');
					printf('</div>');
					break;
				default:
					printf('<div class="jpost-item">');
					printf('<div class=label>%s</div>', $v);
					printf('<div class="ele %s"><input type="text" name="%s" value="%s" placeholder="%s" /></div>', $k, $k, esc_attr($kv), $e);
					printf('</div>');
					break;
			}
		}
		if (get_post_meta(get_the_ID(), '_jpost_task_url', true)) {
			printf('<div class="jpost-item">');
			printf('<div class=label>%s [ <a href="%s">?</a> ]</div>', __('定时'), $this->jpost_readme);
			printf('<div class="ele %s"><a href="%s">%s</a></div>', 'cron', $cron_url, $cron_url);
			printf('</div>');
		}

		echo wp_kses($this->jpost_ads(), $this->allowed_html);
	}

	/**
	 * Metabox 显示
	 * 
	 * @since 2021.5.1
	 */
	public function meta_box_others()
	{
		printf('<input type="hidden" name="_jpost_nonce" value="%s" />', esc_attr(wp_create_nonce(plugin_basename(__FILE__))));
		$key = $this->jpost_keys();
		echo wp_kses($this->jpost_ads(), $this->allowed_html);
		$required = array(
			'_jpost_task_url',
			'_jpost_task_list_url',
			'_jpost_task_list_a_element',
			'_jpost_single_page_title',
			'_jpost_single_page_content'
		);
		$select_format = array('_jpost_task_get_ele_method',);
		$cron_url = $this->replaceSymbolURL(get_the_permalink() . '?&' . http_build_query(array('jpage' => 'auto', 'i' => 'auto')));
		foreach ($key as list($k, $v, $e)) {
			$kv = get_post_meta(get_the_ID(), $k, true);
			switch ($k) {
				case '_jpost_search':
					$replaceAry = get_post_meta(get_the_ID(), '_jpost_replace', true);
					if (is_array($kv)) {
						foreach ($kv as $kk => $vv) {
							printf('<div class="jpost-item"><div class="item-box">');
							printf('<div class="label">%s</div>', $v);
							printf('<div class="ele %s"><textarea name="%s[]" data-tname="%s" data-rname="%s">%s</textarea></div>', $k, $k, __('搜索', 'wp-jpost'), __('替换', 'wp-jpost'), $vv);
							printf('<div class="label label%s">%s</div>', '_jpost_replace', __('替换', 'wp-jpost'));
							printf('<div class="ele %s"><textarea name="%s[]" data-rname="%s">%s</textarea></div>', '_jpost_replace', '_jpost_replace', __('替换', 'wp-jpost'), $replaceAry[$kk]);
							if ($kk == 0) {
								printf('<div class="add_search_replace"><input type="button" name="add_search_replace" class="add_search_replace" value=" + " /></div>');
							} else {
								printf('<div class="del_search_replace"><input type="button" name="del_search_replace" class="del_search_replace" value=" - "></div>');
							}
							printf('</div></div>');
						}
					} else {
						printf('<div class="jpost-item"><div class="item-box">');
						printf('<div class="label">%s</div>', $v);
						printf('<div class="ele %s"><textarea name="%s[]" data-tname="%s" data-rname="%s">%s</textarea></div>', '_jpost_search', '_jpost_search', __('搜索', 'wp-jpost'), __('替换', 'wp-jpost'), '');
						printf('<div class="label label%s">%s</div>', '_jpost_replace', __('替换', 'wp-jpost'));
						printf('<div class="ele %s"><textarea name="%s[]">%s</textarea></div>', '_jpost_replace', '_jpost_replace', '');
						printf('<div class="add_search_replace"><input type="button" name="add_search_replace" class="add_search_replace" value=" + " /></div>');
						printf('</div></div>');
					}
					break;
				case '_jpost_replace':
					break;
				case '_jpost_post_status':
					$statuses = array_reverse(get_post_statuses());
					$jstatus = $kv ? $kv : 'publish';
					printf('<div class="jpost-item">');
					printf('<div class=label>%s</div>', $v);
					printf('<div class="ele %s">', $k);
					printf('<select name="%s" id="%s" value="%s">', $k, $k, $jstatus);
					foreach ($statuses as $i => $j) {
						printf('<option value="%s" %s>%s</option>', $i, $i === $jstatus ? "selected" : '', $j);
					}
					printf('</select> </div></div>');
					break;
				case in_array($k, array(
					'_jpost_auto_save_images',
					'_jpost_autoremove_links',
					'_jpost_coverage_mode',
					'_jpost_baidu_translate',
					'_jpost_skip_error',
					'_jpost_check_login',
					'_jpost_random_in_url'
				)):
					printf('<div class="jpost-item">');
					printf('<div class=label>%s</div>', $v);
					printf('<div class="ele %s"><input type="checkbox" type="text" name="%s" %s></div>', $k, $k, $kv ? 'checked="checked"' : '');
					printf('</div>');
					break;
				case in_array($k, array('_jpost_remove_tags', '_jpost_random_keyword')):
					printf('<div class="jpost-item">');
					if (is_array($kv)) {
						foreach ($kv as $kk => $vv) {
							printf('<div class="item-box">');
							printf('<div class="label">%s</div>', $v);
							printf('<div class="ele %s"><textarea name="%s[]" data-name="%s" data-tname="%s" placeholder="%s" >%s</textarea></div>', $k, $k, $k, $v, $vv, $vv ? $vv : $e);
							if ($kk == 0) {
								printf('<div class="add_tags"><input type="button" name="add_tags" class="add_tags" value=" + " /></div>');
							} else {
								printf('<div class="del_tags"><input type="button" name="del_tags" class="del_tags" value=" - "></div>');
							}
							printf('</div>');
						}
					} else {
						printf('<div class="item-box">');
						printf('<div class="label">%s</div>', $v);
						printf('<div class="ele %s"><textarea name="%s[]" data-name="%s" data-tname="%s">%s</textarea></div>', $k, $k, $k, $v, '');
						printf('<div class="add_tags"><input type="button" name="add_tags" class="%s" value=" + " /></div>', $k, $k);
						printf('</div>');
					}
					printf('</div>');
					break;
				case in_array($k, $required):
					printf('<div class="jpost-item">');
					printf('<div class=label>%s [<span style="color:red;"> * </span>] </div>', $v);
					if (in_array($k, ['_jpost_task_list_a_element'])) {
						if (!is_array($kv)) {/* selector界面 */
							printf(
								'<div class="item-box">
										<div class="ele %s"><input type="text" name="%s[]" value="%s" placeholder="%s" %s style="width:80%%;" data-name="%s" data-tname="%s" />
											<button title="%s" data-popup-field-name class="visual_selector">%s</button>
										</div>
										<div class="add_tags"><input type="button" name="add_a_tags" class="%s" value=" + " /></div>
									</div>',
								$k,
								$k,
								esc_attr($kv),
								$e,
								'required',
								$k,
								$k,
								__('可视化定位', 'wp-jpost'),
								__('可视化定位', 'wp-jpost'),
								$k
							);
						} else {	/* 可视化界面 */
							foreach ($kv as $kk => $vv) {
								printf(
									'<div class="item-box">
											<div class="ele %s"><input type="text" name="%s[]" value="%s" placeholder="%s" %s style="width:80%%;" data-name="%s" data-tname="%s" />
												<button title="%s" data-popup-field-name class="visual_selector">%s</button>
											</div>',
									$k,
									$k,
									esc_attr($vv),
									$e,
									'required',
									$k,
									$k,
									__('可视化定位', 'wp-jpost'),
									__('可视化定位', 'wp-jpost')
								);
								if ($kk == 0) {
									printf('<div class="add_tags"><input type="button" name="add_a_tags" class="%s" value=" + " /></div>', $k);
								} else {
									printf('<div class="del_tags"><input type="button" name="del_tags" class="del_tags" value=" - "></div>');
								}
								printf('</div>');
							}
						}
					} elseif (in_array($k, ['_jpost_single_page_title', '_jpost_single_page_content'])) {
						printf(
							'<div class="ele %s"><input type="text" name="%s" value="%s" placeholder="%s" %s style="width:80%%;"/>
									<button title="%s" data-popup-field-name class="visual_selector">%s</button>
								</div>',
							$k,
							$k,
							esc_attr($kv),
							$e,
							'required',
							__('可视化定位', 'wp-jpost'),
							__('可视化定位', 'wp-jpost')
						);
					} else {
						printf('<div class="ele %s"><input type="text" name="%s" value="%s" placeholder="%s" %s /></div>', $k, $k, esc_attr($kv), $e, 'required');
					}
					printf('</div>');
					break;
				case in_array($k, $select_format):
					printf('<div class="jpost-item">');
					printf('<div class=label>%s [<span style="color:red;"> * </span>] </div>', $v);
					printf('<div class="ele %s"><select name="%s">', $k, $k);
					foreach ($e as $select_val => $select_name) {
						printf('<option value="%s" %s>%s</option>', $select_val, $select_val == $kv ? 'selected' : '', $select_name);
					}
					printf('</select></div>');
					printf('</div>');
					break;
				default:
					printf('<div class="jpost-item">');
					printf('<div class=label>%s</div>', $v);
					printf('<div class="ele %s"><input type="text" name="%s" value="%s" placeholder="%s" /></div>', $k, $k, esc_attr($kv), $e);
					printf('</div>');
					break;
			}
		}
		if (get_post_meta(get_the_ID(), '_jpost_task_url', true)) {
			printf('<div class="jpost-item">');
			printf('<div class=label>%s [ <a href="%s">?</a> ]</div>', __('定时'), $this->jpost_readme);
			printf('<div class="ele %s"><a href="%s">%s</a></div>', 'cron', $cron_url, $cron_url);
			printf('</div>');
		}
	}

	/**
	 * Metabox 保存
	 *
	 * @param  string  $post_id Post Id.
	 * @param  WP_Post $post   Post.
	 * 
	 * @since 2021.5.1
	 */
	public function meta_box_save($post_id, $post)
	{
		$key = $this->jpost_keys();
		// Verify the nonce.
		if (!isset($_POST['_jpost_nonce']) || !wp_verify_nonce($_POST['_jpost_nonce'], plugin_basename(__FILE__))) {
			return;
		}
		// Don't try to save the data under autosave, ajax, or future post.
		if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
			return;
		};
		if (defined('DOING_AJAX') && DOING_AJAX) {
			return;
		};
		if (defined('DOING_CRON') && DOING_CRON) {
			return;
		};
		// Is the user allowed to edit the URL?
		if (!current_user_can('edit_posts') || $this->post_type !== $post->post_type) {
			return;
		}
		foreach ($key as list($k,)) {
			$value = isset($_POST[$k]) ? $this->sanitize_text_or_array_field($_POST[$k]) : '';
			if ($value) {
				// Save/update.
				update_post_meta($post->ID, $k, $value);
			} else {
				// Delete if blank.
				delete_post_meta($post->ID, $k);
			}
		}
	}
	/**
	 * Recursive sanitation for text or array
	 * 
	 * @param $array_or_string (array|string)
	 * @since  0.1
	 * @return mixed
	 */
	public function sanitize_text_or_array_field($array_or_string)
	{
		if (is_string($array_or_string)) {
			$array_or_string = sanitize_text_field($array_or_string);
		} elseif (is_array($array_or_string)) {
			foreach ($array_or_string as $key => &$value) {
				if (is_array($value)) {
					$value = $this->sanitize_text_or_array_field($value);
				} else {
					$value = sanitize_text_field($value);
				}
			}
		}

		return $array_or_string;
	}
	/**
	 * 载入多语言
	 * 
	 * @since 2021.5.1
	 */
	public function load_textdomain()
	{
		load_plugin_textdomain('wp-jpost', false, dirname(dirname(plugin_basename(__FILE__))) . '/languages/');
	}

	/**
	 * 注册文章类型：wp-jpost
	 * 
	 * @since 2021.5.1
	 */
	public function register_post_type()
	{

		$slug = $this->post_type;

		$rewrite_slug_default = $this->rewrite_slug;

		$labels = array(
			'name'               => __('列表', 'wp-jpost'),
			'singular_name'      => __('采集', 'wp-jpost'),
			'add_new'            => __('添加', 'wp-jpost'),
			'add_new_item'       => __('添加任务', 'wp-jpost'),
			'edit'               => __('编辑', 'wp-jpost'),
			'edit_item'          => __('编辑任务', 'wp-jpost'),
			'new_item'           => __('新任务', 'wp-jpost'),
			'view'               => __('查看网址', 'wp-jpost'),
			'view_item'          => __('查看任务', 'wp-jpost'),
			'search_items'       => __('搜索任务', 'wp-jpost'),
			'not_found'          => __('无任务', 'wp-jpost'),
			'not_found_in_trash' => __('垃圾箱中无任务', 'wp-jpost'),
			'messages'           => array(
				0  => '', // Unused. Messages start at index 1.
				/* translators: %s: link for the update */
				1  => __('已更新. <a href="%s">查看</a>', 'wp-jpost'),
				2  => __('自定义字段已更新.', 'wp-jpost'),
				3  => __('自定义字段已删除.', 'wp-jpost'),
				4  => __('已更新.', 'wp-jpost'),
				/* translators: %s: date and time of the revision */
				5  => isset($_GET['revision']) ? sprintf(__('Post restored to revision from %s', 'wp-jpost'), wp_post_revision_title(sanitize_text_field((int) $_GET['revision']), false)) : false, // phpcs:ignore
				/* translators: %s: URL to view */
				6  => __('已更新. <a href="%s">查看</a>', 'wp-jpost'),
				7  => __('已保存.', 'wp-jpost'),
				8  => __('已提交.', 'wp-jpost'),
				9  => __('已安排', 'wp-jpost'),
				10 => __('垃圾箱已更新.', 'wp-jpost'),
			),
		);

		$labels = apply_filters('wp_jpost_labels', $labels);

		$rewrite_slug = apply_filters('WP-JPost-Slug', $rewrite_slug_default);

		$rewrite_slug = sanitize_title($rewrite_slug, $rewrite_slug_default);

		// Ref: https://developer.wordpress.org/reference/functions/add_post_type_support/.
		$supports_array = apply_filters('wp_jposts_post_type_supports', array('title', 'author'));

		// Ref: https://developer.wordpress.org/reference/functions/register_post_type/.
		register_post_type(
			$slug,
			array(
				'labels'              => $labels,
				'public'              => true,
				'exclude_from_search' => apply_filters('jposts_exclude_from_search', true),
				'show_ui'             => true,
				'query_var'           => true,
				'menu_position'       => 999,
				'supports'            => $supports_array,
				'rewrite'             => array(
					'slug'       => $rewrite_slug,
					'with_front' => false,
				),
				'show_in_menu'	=>	'wp-jpost',
				'show_in_rest'        => true,
			)
		);

		register_taxonomy_for_object_type('category', $slug);
		register_taxonomy_for_object_type('post_tag', $slug);
	}



	/**
	 * 构造网络请求参数
	 * 
	 * @param type 类型
	 * 
	 * @since 2021.5.1
	 */
	private function constructURL($type = '')
	{
		$url = get_the_permalink();
		$parm = array();
		$ary = array('continue', 'restart', 'debug');
		if (!in_array($type, $ary)) {
			wp_die('constructURL Error.');
		}
		$jpage = get_post_meta(get_the_ID(), '_jpost_cPage', true);
		$i = get_post_meta(get_the_ID(), '_jpost_i', true);
		if ($type == 'continue' && $jpage) {
			$parm['jpage'] = $jpage;
			$parm['i'] = $i;
		}
		$parm['jpage'] = isset($parm['jpage']) ? $parm['jpage'] : get_post_meta(get_the_ID(), '_jpost_task_list_max_page', true);
		if ($type == 'debug')	$parm['debug'] = 1;
		$url .= '?&' . http_build_query($parm);
		return $this->replaceSymbolURL($url);
	}

	/**
	 * 统计url中？出现的次数，替换符号：?&
	 * 
	 * @param url 网址
	 * 
	 * @return string
	 * 
	 * @since 2021.5.16
	 */
	public function replaceSymbolURL($url)
	{
		$count = substr_count($url, '?');
		if ($count > 1) {
			$url = str_replace('?&', '&', $url);
		} else {
			$url = str_replace('?&', '?', $url);
		}
		return $url;
	}


	/**
	 * 添加 admin 固定栏
	 * 
	 * @since 2021.5.1
	 */
	public function admin_bar_render()
	{
		global $wp_admin_bar;
		$wp_admin_bar->add_menu(
			array(
				'parent' => false, // 'false' 为添加住菜单，也可以输入父级菜单的 ID 
				'id' => 'jpost_debug', // 自定义链接的 ID
				'title' => __('调试任务', 'wp-jpost'), // 自定义链接标题 
				'href' => $this->constructURL('debug'), // 链接地址 
				'meta' => false // 用来设置自定义链接属性选项的一个数组：array( 'html' => '', 'class' => '', 'onclick' => '', target => '', title => '' ); 
			)
		);
		$wp_admin_bar->add_menu(
			array(
				'parent' => false,
				'id' => 'jpost_grab',
				'title' => __('开始任务', 'wp-jpost'),
				'href' => $this->constructURL('restart'),
				'meta' => false
			)
		);
		$wp_admin_bar->add_menu(
			array(
				'parent' => false,
				'id' => 'jpost_grab',
				'title' => __('继续', 'wp-jpost'),
				'href' => $this->constructURL('continue'),
				'meta' => false
			)
		);
	}

	/**
	 * 后台顶部链接显示版本
	 */
	public function jpost_admin_bar()
	{
		global $wp_admin_bar;
		if (current_user_can('manage_options')) {
			$wp_admin_bar->add_menu(array(
				'parent' => 0,
				'id' => 'jpost-page',
				'title' => 'WP-JPOST ',
				'href' => admin_url('edit.php?post_type=' . $this->post_type)
			));
		}
	}

	/**
	 * 广而告之
	 * 
	 * @since 2021.5.1
	 */
	public function jpost_ads()
	{
		return '<div class="jads" style="text-align:center;width:100%;margin:12px auto 12px auto;">
			<a href="https://www.jiloc.com/go/qcloud-act" style="margin-right: 24px;">
				<img src="' . JPOST_URL . '/assets/images/contact.png">
			</a></div>';
	}

	/**
	 * 从服务器获取最新版本
	 * 
	 * @since 2021.5.15
	 * 
	 * @return html
	 */
	// public function versioncheck(){
	// 	$result = wp_remote_request( JPOST_UPDATE_URL );  
	// 	return wp_remote_retrieve_body( $result );
	// }

	/**
	 * 检查更新
	 * 
	 * @since 2021.5.15
	 */
	// public function update(){
	// 	if((int)$this->versioncheck() > (int)*100){
	// 		echo wp_kses('<p>有更新可下载，<a href="https://www.jiloc.com/go/wp-jpost">去下载更新</a></p>','wp-jpost',$this->allowed_html);
	// 	}else{
	// 		echo wp_kses('<p>无更新可下载。</p>',$this->allowed_html);
	// 	}
	// 	echo wp_kses('官方网站：<a href="'.$this->jpost_readme.'">'.$this->jpost_readme.'</a>',$this->allowed_html);	 
	// }
}
